import { useEffect, useState } from 'preact/hooks';
import API_BASE_URL from '../../config';
import './style.css';

export function Admin() {
    const isAdmin = localStorage.getItem('admin') === "true";
    const [users, setUsers] = useState([]);
    const [groups, setGroups] = useState([]);
    const [loading, setLoading] = useState(true);
    const [error, setError] = useState(null);
    const [newUser, setNewUser] = useState({ username: '', password: '', name: '' });
    const [newGroupName, setNewGroupName] = useState('');
    const [selectedGroupId, setSelectedGroupId] = useState('');

    if (!isAdmin) {
        return <div className="no-access">You do not have permission to view this page.</div>;
    }

    useEffect(() => {
        const fetchData = async () => {
            try {
                const usersResponse = await fetch(`${API_BASE_URL}/admin/users`, {
                    method: 'GET',
                    headers: {
                        Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                    }
                });
                const usersData = await usersResponse.json();
                if (!usersResponse.ok) throw new Error('Failed to fetch users');
                setUsers(usersData);

                const groupsResponse = await fetch(`${API_BASE_URL}/admin/group`, {
                    method: 'GET',
                    headers: {
                        Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                    }
                });
                const groupsData = await groupsResponse.json();
                if (!groupsResponse.ok) throw new Error('Failed to fetch groups');
                setGroups(groupsData);
            } catch (err) {
                setError(err.message);
            } finally {
                setLoading(false);
            }
        };

        fetchData();
    }, []);

    const deleteUser = async (userId) => {
        try {
            const response = await fetch(`${API_BASE_URL}/admin/user/${userId}`, {
                method: 'DELETE',
                headers: {
                    Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                }
            });
            if (!response.ok) throw new Error('Failed to delete user');
            setUsers(prevUsers => prevUsers.filter(user => user.id !== userId));
        } catch (error) {
            console.error(error);
        }
    };

    const deleteGroup = async (groupId) => {
        try {
            const response = await fetch(`${API_BASE_URL}/admin/group/${groupId}`, {
                method: 'DELETE',
                headers: {
                    Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                }
            });
            if (!response.ok) throw new Error('Failed to delete group');
            setGroups(prevGroups => prevGroups.filter(group => group.GroupID !== groupId));
            if (selectedGroupId === groupId) setSelectedGroupId('');
        } catch (error) {
            console.error(error);
        }
    };

    const addUserToGroup = async (userId) => {
        try {
            const response = await fetch(`${API_BASE_URL}/admin/user/group/${selectedGroupId}`, {
                method: 'POST',
                headers: {
                    Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                },
                body: JSON.stringify({ "userId": userId })
            });
            if (!response.ok) throw new Error('Failed to add user to group');
        } catch (error) {
            console.error(error);
        }
    };

    const registerUser = async (e) => {
        e.preventDefault();
        try {
            const response = await fetch(`${API_BASE_URL}/admin/register`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                },
                body: JSON.stringify(newUser),
            });
            if (!response.ok) throw new Error('Failed to register user');
            const registeredUser = await response.json();
            setUsers(prevUsers => [...prevUsers, registeredUser]);
            setNewUser({ username: '', password: '', name: '' });
        } catch (error) {
            console.error(error);
            setError(error.message);
        }
    };

    const createGroup = async (e) => {
        e.preventDefault();
        try {
            const response = await fetch(`${API_BASE_URL}/admin/group`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: `Bearer ${localStorage.getItem('accessToken')}`
                },
                body: JSON.stringify({ name: newGroupName }),
            });
            if (!response.ok) throw new Error('Failed to create group');
            const createdGroup = await response.json();
            setGroups(prevGroups => [...prevGroups, createdGroup]);
            setNewGroupName('');
        } catch (error) {
            console.error(error);
            setError(error.message);
        }
    };

    if (loading) return <p>Loading...</p>;
    if (error) return <p>Error: {error}</p>;

    return (
        <div className="admin-dashboard">
            <h1>Admin Dashboard</h1>

            <div className="grid-container">
                <div className="section">
                    <h2>Users</h2>
                    <ul>
                        {users.map(user => (
                            <li key={user.id}>
                                {user.name}
                                <button className="delete-btn" onClick={() => deleteUser(user.id)}>Delete</button>
                            </li>
                        ))}
                    </ul>
                </div>

                <div className="section">
                    <h2>Groups</h2>
                    <ul>
                        {groups.map(group => (
                            <li key={group.id}>
                                {group.GroupName}
                                <button className="delete-btn" onClick={() => deleteGroup(group.GroupID)}>Delete</button>
                            </li>
                        ))}
                    </ul>
                </div>

                <div className="section">
                    <h2>Add User to Group</h2>
                    <form onSubmit={(e) => {
                        e.preventDefault();
                        const userId = e.target.userId.value;
                        if (selectedGroupId) addUserToGroup(userId);
                    }}>
                        <select name="userId">
                            {users.map(user => (
                                <option key={user.id} value={user.id}>{user.name}</option>
                            ))}
                        </select>
                        <select
                            name="groupId"
                            value={selectedGroupId}
                            onChange={(e) => setSelectedGroupId(e.target.value)}
                        >
                            <option value="">Select a group</option>
                            {groups.map(group => (
                                <option key={group.id} value={group.id}>{group.GroupName}</option>
                            ))}
                        </select>
                        <button type="submit">Add User</button>
                    </form>
                </div>

                <div className="section">
                    <h2>Register New User</h2>
                    <form onSubmit={registerUser}>
                        <input
                            type="text"
                            placeholder="Username"
                            value={newUser.username}
                            onChange={(e) => setNewUser({ ...newUser, username: e.target.value })}
                            required
                        />
                        <input
                            type="password"
                            placeholder="Password"
                            value={newUser.password}
                            onChange={(e) => setNewUser({ ...newUser, password: e.target.value })}
                            required
                        />
                        <input
                            type="text"
                            placeholder="Name"
                            value={newUser.name}
                            onChange={(e) => setNewUser({ ...newUser, name: e.target.value })}
                            required
                        />
                        <button type="submit">Register User</button>
                    </form>
                </div>

                <div className="section">
                    <h2>Create New Group</h2>
                    <form onSubmit={createGroup}>
                        <input
                            type="text"
                            placeholder="Group Name"
                            value={newGroupName}
                            onChange={(e) => setNewGroupName(e.target.value)}
                            required
                        />
                        <button type="submit">Create Group</button>
                    </form>
                </div>
            </div>
        </div>
    );
}
